﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing.Printing;

using VidyaVitamin.Plugin;

namespace VidyaVitamin.Plugin.Thumbnail
{
	public partial class PrintForm : Form
	{
		private string pct_name;
		private ThumbnailData[] tdata;
		private List<int> sellist;
		private IFirebirdTable table;
		private IVVUtility util;
		private bool embed;

		public PrintForm(IFirebirdTable table, string pct_name, 
			ThumbnailData[] tdata, List<int> sellist, IVVUtility util)
		{
			InitializeComponent();

			lblNumCards.Text = string.Format(Properties.Resources.strNumCards, sellist.Count);

			this.util = util;
			this.pct_name = pct_name;
			this.tdata = tdata;
			this.sellist = sellist;
			this.table = table;
			embed = table.IFirebird.ExistTable(table.TableName + "$" + pct_name);
		}

		private void btnPageSetting_Click(object sender, EventArgs e)
		{
			page.ShowDialog();
		}

		private void btnPrint_Click(object sender, EventArgs e)
		{
			if (print.ShowDialog() == DialogResult.Cancel) return;
			p = 0;
			pdoc.Print();
		}

		private void btnPreview_Click(object sender, EventArgs e)
		{
			p = 0;
			preview.ClientSize = new Size(600, 800);
			preview.ShowDialog();
		}

		private int p = 0;

		private int CentiToPixel(float cm, float dpi)
		{
			return (int)(cm * dpi / 2.54f);
		}

		private int InchToPixel(int inch100, float dpi)
		{
			return (int)(dpi * (float)inch100 / 100.0f);
		}

		private void pdoc_PrintPage(object sender, PrintPageEventArgs e)
		{
			Graphics g = e.Graphics;
			g.PageUnit = GraphicsUnit.Pixel;

			Margins m;
			if (chkMargin.Checked == false) {
				m = (Margins)page.PageSettings.Margins.Clone();
				m.Top = InchToPixel(m.Top, g.DpiY);
				m.Bottom = InchToPixel(m.Bottom, g.DpiY);
				m.Left = InchToPixel(m.Left, g.DpiY);
				m.Right = InchToPixel(m.Right, g.DpiY);
			} else {
				m = new Margins(0, 0, 0, 0);
			}

			Font font = null; StringFormat sf = StringFormat.GenericDefault;
			bool with_title = chkTitle.Checked;
			int title_height = 0;
			if (with_title == true) {
				title_height = (int)CentiToPixel(0.8f, g.DpiY);	// 0.8cm
				font = new Font(FontFamily.GenericSansSerif, CentiToPixel(0.6f, g.DpiY), GraphicsUnit.Pixel);
				sf.Alignment = StringAlignment.Center;
				sf.LineAlignment = StringAlignment.Center;
			}

			int pwidth = InchToPixel(page.PageSettings.PaperSize.Width, g.DpiY);
			int pheight = InchToPixel(page.PageSettings.PaperSize.Height, g.DpiY);
			if (page.PageSettings.Landscape == true) {
				int t = pwidth;
				pwidth = pheight;
				pheight = t;
			}

			int twidth = pwidth - m.Right - m.Left;
			int theight = pheight - m.Top - m.Bottom;

			int horz = (int)numHorz.Value, vert = (int)numVert.Value;
			int thumbs_page = horz * vert;
			int thumb_margin = CentiToPixel(0.2f, g.DpiY);

			string excmd = string.Empty;
			Size tsize = new Size(pwidth / horz, pheight / vert);
			if(tsize.Width < 256 || tsize.Height < 256){
				excmd = Thumbnail.STR_THUMB;
			}else{
				excmd = Thumbnail.STR_BINARY;
			}

			string sqlcmd, str_title = null;
			bool print_all_cards = false;
			int selcount = sellist.Count;
			for (int y = 0; y < vert; y++) {
				if (e.Cancel == true || print_all_cards == true) break;
				int yy = y * theight / vert;
				for (int x = 0; x < horz; x++) {
					if(e.Cancel == true) break;

					int xx = x * twidth / horz;
					Rectangle rc = new Rectangle(m.Left + xx, m.Top+ yy,
						twidth / horz, theight / vert);

					int spos = thumbs_page * p + y * horz + x;
					if (selcount <= spos) { print_all_cards = true; break; }

					int index = tdata[sellist[spos]].index;

					// 画像の取得
					if(with_title == true){
						IList<DBAttributedData> ld = table.GetAttributedValue(DBAttrType.Title, index, index);
						if(string.IsNullOrEmpty(ld[0].title) == false){
							str_title = ld[0].title;
						}else{
							str_title = index.ToString();
						}
					}

					Image img = null;
					MemoryStream ms = null;
					if (embed == true) {
						try{
							sqlcmd = string.Format(
								"SELECT {0} FROM {1}${2} WHERE {3}={4};",
								excmd, table.TableName, pct_name,
								Thumbnail.STR_ID, index);
							byte[] buf = table.IFirebird.ExecScalar(sqlcmd) as byte[];
							if(buf != null){
								ms = new MemoryStream(buf);
								img = Image.FromStream(ms);
								//ms.Close();
							}
						}catch(Exception){}
					} else {
						sqlcmd = string.Format("SELECT {0} FROM {1} WHERE {2}={3};",
							pct_name, table.TableName, Thumbnail.STR_ID, index);
						string path = (string)table.IFirebird.ExecScalar(sqlcmd);
						path = util.ToAbsolutePath(table.EditPath, path);
						try {
							img = Image.FromFile(path);
						} catch (Exception) {
							img = util.ExtractImage(path, tsize);
						}
					}

					rc.Height -= title_height;
					if (img != null) {
						if (vert == 1 && horz == 1 && str_title == null) {
							int aspect1 = (int)((float)img.Width / (float)img.Height);
							int aspect2 = (int)((float)twidth / (float)theight);
							if(aspect1 != aspect2) img.RotateFlip(RotateFlipType.Rotate270FlipNone);
						}

						Rectangle drc = util.GetPictureRect(rc, img.Size);
						if (vert > 1 || horz > 1) drc.Inflate(-thumb_margin, -thumb_margin);

						g.DrawImage(img, drc, new Rectangle(0, 0, img.Width, img.Height), GraphicsUnit.Pixel);

						img.Dispose();
						if (ms != null) ms.Close();
					}
					if (str_title != null) {
						Rectangle drc = rc; drc.Y = rc.Bottom + 1; drc.Height = title_height;
						g.DrawString(str_title, font, Brushes.Black, drc, sf); 
					}
				}
			}

			if (selcount <= thumbs_page * (p + 1)) {
				e.HasMorePages = false;
			} else {
				p++;
				e.HasMorePages = true;
			}
		}
	}
}
